# SPDX-FileCopyrightText: 2024-present Sebastian Tobie <sebastian@sebastian-tobie.de>
#
# SPDX-License-Identifier: MIT
import json
import logging
import os.path
import threading
from logging.config import dictConfig, fileConfig

from gunicorn.glogging import CONFIG_DEFAULTS
from gunicorn.glogging import Logger as gLogger

__version__ = "0.0.3"


class Logger(gLogger):
    def __init__(self, cfg):
        self.error_log = logging.getLogger("gunicorn.error")
        self.access_log = logging.getLogger("gunicorn.access")
        self.logfile = None
        self.lock = threading.Lock()
        self.cfg = cfg
        self.setup(cfg)

    def setup(self, cfg):
        if cfg.logconfig_dict:
            config = CONFIG_DEFAULTS.copy()
            config.update(cfg.logconfig_dict)
            try:
                dictConfig(config)
            except (AttributeError, ImportError, ValueError, TypeError) as exc:
                raise RuntimeError(str(exc))
        elif cfg.logconfig_json:
            config = CONFIG_DEFAULTS.copy()
            if os.path.exists(cfg.logconfig_json):
                try:
                    config_json = json.load(open(cfg.logconfig_json))
                    config.update(config_json)
                    dictConfig(config)
                except (
                    json.JSONDecodeError,
                    AttributeError,
                    ImportError,
                    ValueError,
                    TypeError,
                ) as exc:
                    raise RuntimeError(str(exc))
        elif cfg.logconfig:
            if os.path.exists(cfg.logconfig):
                defaults = CONFIG_DEFAULTS.copy()
                defaults["__file__"] = cfg.logconfig
                defaults["here"] = os.path.dirname(cfg.logconfig)
                fileConfig(cfg.logconfig, defaults=defaults, disable_existing_loggers=False)
            else:
                msg = "Error: log config '%s' not found"
                raise RuntimeError(msg % cfg.logconfig)

    def access(self, resp, req, environ, request_time):
        """See http://httpd.apache.org/docs/2.0/logs.html#combined
        for format details
        """

        if not (
            self.cfg.accesslog
            or self.cfg.logconfig
            or self.cfg.logconfig_dict
            or self.cfg.logconfig_json
            or (self.cfg.syslog and not self.cfg.disable_redirect_access_to_syslog)
        ):
            return

        # wrap atoms:
        # - make sure atoms will be test case insensitively
        # - if atom doesn't exist replace it by '-'
        safe_atoms = self.atoms_wrapper_class(self.atoms(resp, req, environ, request_time))
        extra = dict(
            method=environ.get("REQUEST_METHOD"),
            path=environ.get("PATH_INFO"),
            query=environ.get("QUERY_STRING"),
            proto=environ.get("SERVER_PROTOCOL"),
            time="%d.%06d" % (request_time.seconds, request_time.microseconds),
        )
        if environ.get("REMOTE_ADDR", False):
            extra["remote"] = environ.get("REMOTE_ADDR")
        if getattr(resp, "sent", False):
            extra["length"] = getattr(resp, "sent")
        if environ.get("HTTP_REFERER", False):
            extra["referer"] = environ.get("HTTP_REFERER")
        if environ.get("HTTP_USER_AGENT", False):
            extra["user_agent"] = environ.get("HTTP_USER_AGENT")

        try:
            self.access_log.info(self.cfg.access_log_format, safe_atoms, extra=extra)
        except Exception:
            self.error(traceback.format_exc())
